@@ -29,7 +29,7 @@ from trl import AutoModelForCausalLMWithValueHead, AutoModelForSeq2SeqLMWithValu
 from trl.core import respond_to_batch
 
 from .testing_constants import CI_HUB_ENDPOINT, CI_HUB_USER, CI_HUB_USER_TOKEN
-from .testing_utils import require_peft
+from .testing_utils import require_peft, require_torch_multi_gpu
 
 
 EXPECTED_STATS = [
@@ -778,3 +778,70 @@ class PPOTrainerTester(unittest.TestCase):
                     ],
                 )
             )
+
+    @require_peft
+    @require_torch_multi_gpu
+    @mark.peft_test
+    def test_peft_model_ppo_trainer_multi_gpu(self):
+        from peft import LoraConfig, get_peft_model
+        from transformers import AutoModelForCausalLM
+
+        lora_config = LoraConfig(
+            r=16,
+            lora_alpha=32,
+            lora_dropout=0.05,
+            bias="none",
+            task_type="CAUSAL_LM",
+        )
+        gpt2_model = AutoModelForCausalLM.from_pretrained(
+            "gpt2", device_map="balanced", max_memory={0: "500MB", 1: "500MB"}
+        )
+
+        self.assertTrue(set(gpt2_model.hf_device_map.values()) == {0, 1})
+
+        # this line is very important
+        def make_inputs_require_grad(module, input, output):
+            output.requires_grad_(True)
+
+        gpt2_model.get_input_embeddings().register_forward_hook(make_inputs_require_grad)
+
+        peft_model = get_peft_model(gpt2_model, lora_config)
+        model = AutoModelForCausalLMWithValueHead.from_pretrained(peft_model)
+
+        self.assertTrue(model.is_sequential_parallel)
+
+        dummy_dataset = self._init_dummy_dataset()
+        self.ppo_config.batch_size = 2
+        self.ppo_config.mini_batch_size = 1
+
+        ppo_trainer = PPOTrainer(
+            config=self.ppo_config,
+            model=model,
+            ref_model=None,
+            tokenizer=self.gpt2_tokenizer,
+            dataset=dummy_dataset,
+        )
+
+        self.assertTrue(ppo_trainer.ref_model is None)
+
+        dummy_dataloader = ppo_trainer.dataloader
+
+        # train model with ppo
+        for query_tensor, response_tensor in dummy_dataloader:
+            # define a reward for response
+            # (this could be any reward such as human feedback or output from another model)
+            reward = [torch.tensor(1.0), torch.tensor(0.0)]
+            # train model by running a step twice
+            _ = ppo_trainer.step([q for q in query_tensor], [r for r in response_tensor], reward)
+
+            ppo_trainer.model.train()
+            ppo_trainer.model.gradient_checkpointing_enable()
+            _ = ppo_trainer.step([q for q in query_tensor], [r for r in response_tensor], reward)
+            break
+
+        # check gradients
+        for name, param in model.named_parameters():
+            if "lora" in name or "v_head" in name:
+                self.assertTrue(param.grad is not None, f"Parameter {name} has a no gradient")
+            else:
+                self.assertTrue(param.grad is None, f"Parameter {name} has a gradient")
